package javax.swing;

import java.lang.ref.WeakReference;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.beans.PropertyChangeEvent;
import java.lang.ref.ReferenceQueue;
import java.io.Serializable;
import java.beans.PropertyChangeListener;

abstract class ActionPropertyChangeListener<T extends JComponent> implements PropertyChangeListener, Serializable
{
    private static ReferenceQueue<JComponent> queue;
    private transient ActionPropertyChangeListener.OwnedWeakReference<T> target;
    private Action action;
    
    private static ReferenceQueue<JComponent> getQueue() {
        synchronized (ActionPropertyChangeListener.class) {
            if (ActionPropertyChangeListener.queue == null) {
                ActionPropertyChangeListener.queue = new ReferenceQueue();
            }
        }
        return ActionPropertyChangeListener.queue;
    }
    
    public ActionPropertyChangeListener(T target, Action action) {
        this.setTarget(target);
        this.action = action;
    }
    
    @Override
    public final void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        JComponent target = this.getTarget();
        if (target == null) {
            this.getAction().removePropertyChangeListener(this);
        }
        else {
            this.actionPropertyChanged(target, this.getAction(), propertyChangeEvent);
        }
    }
    
    protected abstract void actionPropertyChanged(T p0, Action p1, PropertyChangeEvent p2);
    
    private void setTarget(T t) {
        ReferenceQueue<JComponent> queue = getQueue();
        ActionPropertyChangeListener.OwnedWeakReference<Object> ownedWeakReference;
        while ((ownedWeakReference = (ActionPropertyChangeListener.OwnedWeakReference)queue.poll()) != null) {
            ActionPropertyChangeListener<Object> owner = ownedWeakReference.getOwner();
            Action action = owner.getAction();
            if (action != null) {
                action.removePropertyChangeListener(owner);
            }
            else {
                continue;
            }
        }
        this.target = new ActionPropertyChangeListener.OwnedWeakReference(t, queue, this);
    }
    
    public T getTarget() {
        if (this.target == null) {
            return null;
        }
        return (T)this.target.get();
    }
    
    public Action getAction() {
        return this.action;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeObject(this.getTarget());
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        JComponent target = (JComponent)objectInputStream.readObject();
        if (target != null) {
            this.setTarget(target);
        }
    }
    
    private static class OwnedWeakReference<U extends JComponent> extends WeakReference<U>
    {
        private ActionPropertyChangeListener<?> owner;
        
        OwnedWeakReference(U referent, ReferenceQueue<? super U> q, ActionPropertyChangeListener<?> owner) {
            super(referent, q);
            this.owner = owner;
        }
        
        public ActionPropertyChangeListener<?> getOwner() {
            return this.owner;
        }
    }
}
